<?php
/**
 * Manager to help with getting values from a configuration array.
 *
 * @copyright   2013-2017 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2017.2/1557039
 */

namespace Application\Config;

class ConfigManager
{
    const FILES_DOWNLOAD_TIMEOUT = 'files.download_timeout';
    const REVIEWS_EXPAND_ALL     = 'reviews.expand_all_file_limit';
    const DIFF_MAX_DIFFS         = 'diffs.max_diffs';
    const FILES_MAX_SIZE         = 'files.max_size';
    const INT                    = 'int';

    // Metadata to define types etc (structure should agree with the configuration array)
    private static $configMetaData = array(
        'reviews' => array(
            'expand_all_file_limit' => array('type' => self::INT),
        ),
        'diffs'   => array(
            'max_diffs'             => array('type' => self::INT)
        ),
        'files'   => array(
            'max_size'              => array('type' => self::INT),
            'download_timeout'      => array('type' => self::INT)
    ));

    /**
     * Gets a value from the config and checks if it is valid. Simple case at the moment - we may
     * want to enhance to add ranges etc.
     * @param $config the config data
     * @param $path the array path (for nested values use dot notation, for example 'reviews.expand_all_file_limit'.
     * @param $metaData optional metadata override
     * @param $default optional default value that will be returned without validation if any problems occur
     * @return mixed|null
     */
    public static function getValue($config, $path, $default = null, $metaData = null)
    {
        // This will throw an exception if the value is not set
        $configValue = null;
        try {
            $configValue = self::getValueFromConfig($config, $path);
            try {
                $metaDataElement = self::getValueFromConfig($metaData ? $metaData : self::$configMetaData, $path);
            } catch (ConfigException $e) {
                // Something is wrong with our metadata, perhaps a path is asked for that we haven't yet catered for.
                // Don't fail because of this.
                return $configValue;
            }
            // This will throw an exception if validation fails
            self::validate($configValue, $metaDataElement, $path);
        } catch (ConfigException $ce) {
            if ($default === null) {
                throw $ce;
            } else {
                $configValue = $default;
            }
        }
        return $configValue;
    }

    private static function validate($value, $metaDataValue, $path)
    {
        // Validate all parameters here - currently just type is specified
        $type           = $metaDataValue['type'];
        $convertedValue = null;
        switch ($type) {
            case self::INT:
                if (ctype_digit(strval($value))) {
                    $convertedValue = intval($value);
                }
                break;
        }
        if ($convertedValue === null) {
            throw new ConfigException(
                "Value '" . $value . "' at path '" . $path . "' is invalid"
            );
        }
        return $convertedValue;
    }

    /**
     * Iterates through the configuration to find a value.
     * @param $config the config
     * @param $path the array path (for nested values use dot notation, for example 'reviews.expand_all_file_limit'.
     * @return mixed
     * @throws \Exception if the path does not exist or the configuration being searched is not an array.
     */
    private static function getValueFromConfig($config, $path)
    {
        $ref  = &$config;
        $keys = explode('.', $path);
        foreach ($keys as $idx => $key) {
            if (!is_array($ref)) {
                throw new ConfigException('Configuration is not an array');
            }
            if (!array_key_exists($key, $ref)) {
                throw new ConfigException("Path '" . $path . "' does not exist");
            }
            $ref = &$ref[$key];
        }
        return $ref;
    }
}
